#ifndef USE_GRAPHICS_H
#define USE_GRAPHICS_H

// simple, portable graphics
#include "capd/krak/krak.h"

/** helper function for painting */
void paintAxes(Frame& frame, double minx, double miny, double maxx, double maxy, double origx, double origy, double secy){
	double x = (double)((int)minx);
	while(x > minx && x < maxx){
		frame.line(x,miny,x,maxy,YELLOW);
		x += 1.0;
	}
	
	double y = (double)((int)miny);
	while(y > miny && y < maxy){
		frame.line(minx,y,maxx,y,YELLOW);
		y += 1.0;
	}	

	if (origx >= minx && origx <= maxx)
		frame.line(origx,miny,origx,maxy,BLACK);
	if (origy >= miny && origy <= maxy)
		frame.line(minx,origy,maxx,origy,BLACK);
	
	if (secy >= miny && secy <= maxy)
		frame.line(minx,secy,maxx,secy,BLUE);			
}

class Keypress{
public:
	std::string s;
	Keypress(std::string str) : s(str) {}
};

/** draws Rect2Set with applying coordinate change */
void drawSetEx(Frame& frame, const BaseRect2Set& set, int color = GREEN){

	IntervalMatrix C = set.get_C();
	IntervalVector x = set.get_x();
	IntervalVector r = set.get_r0();

	interval dat1[] = {r[0].leftBound(), r[1].rightBound()};
	IntervalVector r_ul = IntervalVector(2, dat1);
	interval dat2[] = {r[0].leftBound(), r[1].leftBound()};
	IntervalVector r_ur = IntervalVector(2, dat2);
	interval dat3[] = {r[0].rightBound(), r[1].rightBound()};
	IntervalVector r_dl = IntervalVector(2, dat3);
	interval dat4[] = {r[0].rightBound(), r[1].leftBound()};
	IntervalVector r_dr = IntervalVector(2, dat4);       

	double poly[8];
	poly[0] = x[0].leftBound() + (C * r_ul)[0].leftBound();
	poly[1] = x[1].leftBound() + (C * r_ul)[1].leftBound();

	poly[2] = x[0].leftBound() + (C * r_ur)[0].leftBound();
	poly[3] = x[1].leftBound() + (C * r_ur)[1].leftBound();

	poly[4] = x[0].leftBound() + (C * r_dr)[0].leftBound();
	poly[5] = x[1].leftBound() + (C * r_dr)[1].leftBound();

	poly[6] = x[0].leftBound() + (C * r_dl)[0].leftBound();
	poly[7] = x[1].leftBound() + (C * r_dl)[1].leftBound();

	for (int i = 0; i < 4; i++){
		frame.line(poly[2 * i], poly[2 * i + 1], poly[(2 * i + 2) % 8], poly[(2 * i + 3) % 8], color);
	}
	frame.dot(poly[0], poly[1], color);
	frame.dot(poly[2], poly[3], color);
	frame.dot(poly[4], poly[5], color);
	frame.dot(poly[6], poly[7], color);

}


/** draws Rect2Set with applying coordinate change */
void drawSetEx(Frame& frame, const Rect2SetParam& set, int color = GREEN){

	IntervalMatrixParam CC = set.get_C();
	IntervalVectorParam xx = set.get_x();
	IntervalVectorParam rr = set.get_r0();
	
	IntervalMatrix C = IntervalMatrix::Identity(2);
	C(1,1) = CC(1,1);
	C(1,2) = CC(1,2);
	C(2,1) = CC(2,1);
	C(2,2) = CC(2,2);
	
	interval datrr[] = {rr[0], rr[1]};
	IntervalVector r = IntervalVector(2, datrr);
	interval datxx[] = {xx[0], xx[1]};
	IntervalVector x = IntervalVector(2, datxx);

	interval dat1[] = {r[0].leftBound(), r[1].rightBound()};
	IntervalVector r_ul = IntervalVector(2, dat1);
	interval dat2[] = {r[0].leftBound(), r[1].leftBound()};
	IntervalVector r_ur = IntervalVector(2, dat2);
	interval dat3[] = {r[0].rightBound(), r[1].rightBound()};
	IntervalVector r_dl = IntervalVector(2, dat3);
	interval dat4[] = {r[0].rightBound(), r[1].leftBound()};
	IntervalVector r_dr = IntervalVector(2, dat4);       

	double poly[8];
	poly[0] = x[0].leftBound() + (C * r_ul)[0].leftBound();
	poly[1] = x[1].leftBound() + (C * r_ul)[1].leftBound();

	poly[2] = x[0].leftBound() + (C * r_ur)[0].leftBound();
	poly[3] = x[1].leftBound() + (C * r_ur)[1].leftBound();

	poly[4] = x[0].leftBound() + (C * r_dr)[0].leftBound();
	poly[5] = x[1].leftBound() + (C * r_dr)[1].leftBound();

	poly[6] = x[0].leftBound() + (C * r_dl)[0].leftBound();
	poly[7] = x[1].leftBound() + (C * r_dl)[1].leftBound();

	for (int i = 0; i < 4; i++){
		frame.line(poly[2 * i], poly[2 * i + 1], poly[(2 * i + 2) % 8], poly[(2 * i + 3) % 8], color);
	}
	frame.dot(poly[0], poly[1], color);
	frame.dot(poly[2], poly[3], color);
	frame.dot(poly[4], poly[5], color);
	frame.dot(poly[6], poly[7], color);

}

template<typename VT>
void drawVector(Frame& frame, const VT& vector, int color = GREEN){
	double x1, x2, y1, y2;
	x1 = vector[0].leftBound();
	y1 = vector[1].leftBound();
	x2 = vector[0].rightBound();
	y2 = vector[1].rightBound();			

	frame.boxFill(x1, y1, x2, y2, color);
	frame.box(x1, y1, x2, y2, color);
	
	frame.dot(x1, y1, color);
	frame.dot(x1, y2, color);
	frame.dot(x2, y1, color);
	frame.dot(x2, y2, color);
}

/** draws interval enclosure of set */
void drawSet(Frame& frame, BaseRect2Set& set, int color = GREEN){
	IntervalVector setAsInterval(set);
	drawVector(frame, setAsInterval, color);
}

class Color{
public:
	int color;
	
	Color(int c){
		color = c;
	}
	
	static Color red;
	static Color green;
	static Color pine;
	static Color orange;
	static Color blue;
	static Color yellow;
};

Color Color::red(RED);
Color Color::green(GREEN);
Color Color::blue(BLUE);
Color Color::orange(ORANGE);
Color Color::pine(OLIVE);
Color Color::yellow(YELLOW);

class KrakOutput{
	Frame frames[3];
	Frame* frame;
	int foreground;
	int background;
public:
	KrakOutput(){
		frame = 0;
		openGW(1200, 800, WHITE, BLACK);
		rootFrame.Clear();
	}
	
	void setup(int frameNum, 
				double minx, 
				double miny, 
				double maxx, 
				double maxy, 
				double origx, 
				double origy, 
				double secy){
		if (frameNum == 0){
			frames[0] = Frame(0, 0, 600, 800, WHITE, BLACK);
			frames[0].setWorldCoord(minx, miny, maxx, maxy);
			paintAxes(frames[0], minx, miny, maxx, maxy, origx, origy, secy);
		}else{
			frames[frameNum] = Frame(600, 400 * ((frameNum-1) % 2), 1200, 400 * (1 + (frameNum-1) % 2), WHITE, BLACK);
			frames[frameNum].setWorldCoord(minx, miny, maxx, maxy);
			paintAxes(frames[frameNum], minx, miny, maxx, maxy, origx, origy, secy);
			//paint the box
			frames[frameNum].box(minx, miny, maxx, maxy, MAGENTA + (frameNum-1) % 2);
			frames[0].box(minx, miny, maxx, maxy, MAGENTA + (frameNum-1) % 2);
		}	
		frame = &(frames[frameNum]);
	}	
	KrakOutput& operator<<(const int& item){
		frame = &frames[item];
		return *this;
	}
	KrakOutput& operator<<(const Color& item){
		foreground = item.color;
		return *this;
	}	
	KrakOutput& operator<<(const char* item){
		(*frame) << item;				
		return *this;
	}
	KrakOutput& operator<<(const BaseRect2Set& item){
		drawSetEx((*frame), item, foreground);
		return *this;
	}
	
	KrakOutput& operator<<(const Rect2SetParam& item){
		drawSetEx((*frame), item, foreground);
		return *this;
	}	
	
	KrakOutput& operator<<(const C1Rect2Set& item){
		IntervalVector itemV(item);
		drawVector((*frame), itemV, foreground);
		return *this;
	}	
	
	KrakOutput& operator<<(const IntervalVectorParam& item){
		drawVector((*frame), item, foreground);
		return *this;
	}	
	
	KrakOutput& operator<<(const IntervalVector& item){
		drawVector((*frame), item, foreground);
		return *this;
	}
	
	KrakOutput& operator<<(const StalkedRect2Set<KrakOutput>& item){
		drawSetEx((*frame), (BaseRect2Set)item, foreground);
		return *this;
	}
	
	KrakOutput& operator<<(const StalkedC1Rect2SetParam<KrakOutput>& item){
		IntervalVectorParam itemV = IntervalVectorParam(BaseC1Rect2SetParam(item));
		drawVector((*frame), itemV, foreground);
		return *this;
	}
		
	KrakOutput& operator<<(const Keypress& item){
		(*frame) << item.s;
		waitBt();
		return *this;		
	}		
	
	template<typename T>
	KrakOutput& operator<<(const T& item){
		(*frame) << item;
		return *this;		
	}	
};

#endif
